Class: Fontisan::Models::InstanceInfo

Inherits:
Lutaml::Model::Serializable
  • Object
show all
Defined in:
lib/fontisan/models/variable_font_info.rb

Overview

InstanceInfo model represents a named instance in a variable font

Each instance defines a predefined combination of axis values, representing a named style/weight/width combination.