Class: Fontisan::Ufo::Compile::FeatureWriters::Mkmk

Inherits:
MarkFamilyBase show all
Defined in:
lib/fontisan/ufo/compile/feature_writers/mkmk.rb

Overview

GPOS MarkToMark (lookup type 6) — mark-to-mark attachment.

Same shape as MarkToBase, but pairs marks with OTHER marks rather than marks with bases. UFO convention: a mark glyph that can attach to another mark has a _<name>mkmk anchor (the attach-FROM point), and the "base mark" it attaches to has the matching <name>mkmk anchor (the attach-TO point).

Pairs every glyph with a _<name>mkmk anchor with every glyph that has the matching <name>mkmk anchor.

Constant Summary collapse

LOOKUP_TYPE =
6
FEATURE_TAG =
"mkmk"
MKMK_SUFFIX =
"mkmk"

Instance Attribute Summary

Attributes inherited from Base

#font

Method Summary

Methods inherited from MarkFamilyBase

#write

Methods inherited from Base

#initialize, #write

Constructor Details

This class inherits a constructor from Fontisan::Ufo::Compile::FeatureWriters::Base