Class: Fontisan::Models::FontInfo
- Inherits:
-
Lutaml::Model::Serializable
- Object
- Lutaml::Model::Serializable
- Fontisan::Models::FontInfo
- Defined in:
- lib/fontisan/models/font_info.rb
Overview
FontInfo model represents comprehensive font metadata extracted from various font tables (name, head, OS/2, etc.)
This model provides a unified interface for accessing font information and supports serialization to YAML and JSON formats through lutaml-model.