Class: Fontisan::Models::OpticalSizeInfo

Inherits:
Lutaml::Model::Serializable
  • Object
show all
Defined in:
lib/fontisan/models/optical_size_info.rb

Overview

OpticalSizeInfo model represents optical size information from a font

Optical size information indicates the design size range for which a font is optimized. This can come from the OS/2 table (version 5+) or from the GPOS ‘size’ feature.