Class: Fontisan::Tables::VariationCommon::RegionAxisCoordinates
- Inherits:
-
Binary::BaseRecord
- Object
- BinData::Record
- Binary::BaseRecord
- Fontisan::Tables::VariationCommon::RegionAxisCoordinates
- Defined in:
- lib/fontisan/tables/variation_common.rb
Overview
Variation region in design space
A region is defined by ranges on one or more axes. Each region has a scalar value (0.0 to 1.0) that determines how much its deltas contribute based on the current design coordinates.
Instance Method Summary collapse
-
#end_value ⇒ Float
Convert end coordinate from F2DOT14 to float.
-
#peak ⇒ Float
Convert peak coordinate from F2DOT14 to float.
-
#start ⇒ Float
Convert start coordinate from F2DOT14 to float.
Methods inherited from Binary::BaseRecord
Instance Method Details
#end_value ⇒ Float
Convert end coordinate from F2DOT14 to float
42 43 44 |
# File 'lib/fontisan/tables/variation_common.rb', line 42 def end_value f2dot14_to_float(end_coord) end |
#peak ⇒ Float
Convert peak coordinate from F2DOT14 to float
35 36 37 |
# File 'lib/fontisan/tables/variation_common.rb', line 35 def peak f2dot14_to_float(peak_coord) end |
#start ⇒ Float
Convert start coordinate from F2DOT14 to float
28 29 30 |
# File 'lib/fontisan/tables/variation_common.rb', line 28 def start f2dot14_to_float(start_coord) end |