Class: Fontisan::Tables::VariationCommon::RegionAxisCoordinates

Inherits:
Binary::BaseRecord
  • Object
show all
Defined in:
lib/fontisan/tables/variation_common.rb

Overview

Variation region in design space

A region is defined by ranges on one or more axes. Each region has a scalar value (0.0 to 1.0) that determines how much its deltas contribute based on the current design coordinates.

Instance Method Summary collapse

Methods inherited from Binary::BaseRecord

#raw_data, read, #valid?

Instance Method Details

#end_valueFloat

Convert end coordinate from F2DOT14 to float

Returns:

  • (Float)

    End coordinate value



43
44
45
# File 'lib/fontisan/tables/variation_common.rb', line 43

def end_value
  f2dot14_to_float(end_coord)
end

#peakFloat

Convert peak coordinate from F2DOT14 to float

Returns:

  • (Float)

    Peak coordinate value



36
37
38
# File 'lib/fontisan/tables/variation_common.rb', line 36

def peak
  f2dot14_to_float(peak_coord)
end

#startFloat

Convert start coordinate from F2DOT14 to float

Returns:

  • (Float)

    Start coordinate value



29
30
31
# File 'lib/fontisan/tables/variation_common.rb', line 29

def start
  f2dot14_to_float(start_coord)
end