Class: Fontisan::Type1::Encodings::Unicode
- Defined in:
- lib/fontisan/type1/encodings.rb
Overview
Unicode Encoding
Uses the Adobe Glyph List to map Unicode codepoints to glyph names. This encoding supports all Unicode characters through the AGL.
Class Method Summary collapse
-
.all_glyph_names ⇒ Array<String>
Get all glyph names in AGL.
-
.codepoint_for_glyph(name) ⇒ Integer?
Get Unicode codepoint for glyph name.
-
.encoding_name ⇒ String
Get encoding name.
-
.glyph_name_for_code(codepoint) ⇒ String
Get glyph name for Unicode codepoint.
-
.include?(_name) ⇒ Boolean
Check if glyph name is in encoding (always true for Unicode).
Class Method Details
.all_glyph_names ⇒ Array<String>
Get all glyph names in AGL
691 692 693 |
# File 'lib/fontisan/type1/encodings.rb', line 691 def self.all_glyph_names AGL.all_glyph_names end |
.codepoint_for_glyph(name) ⇒ Integer?
Get Unicode codepoint for glyph name
669 670 671 |
# File 'lib/fontisan/type1/encodings.rb', line 669 def self.codepoint_for_glyph(name) AGL.unicode_for_glyph_name(name) end |
.encoding_name ⇒ String
Get encoding name
684 685 686 |
# File 'lib/fontisan/type1/encodings.rb', line 684 def self.encoding_name "Unicode" end |
.glyph_name_for_code(codepoint) ⇒ String
Get glyph name for Unicode codepoint
661 662 663 |
# File 'lib/fontisan/type1/encodings.rb', line 661 def self.glyph_name_for_code(codepoint) AGL.glyph_name_for_unicode(codepoint) end |
.include?(_name) ⇒ Boolean
Check if glyph name is in encoding (always true for Unicode)
677 678 679 |
# File 'lib/fontisan/type1/encodings.rb', line 677 def self.include?(_name) true end |