Exception: Fontisan::Type1::SeacReferenceError
- Defined in:
- lib/fontisan/type1/seac_expander.rb
Overview
Error raised when a seac reference cannot be resolved (the referenced glyph is missing, or a cycle is detected in nested seac).
Error raised when a seac reference cannot be resolved (the referenced glyph is missing, or a cycle is detected in nested seac).